/******************************************************************/
/*                                                                */
/*                    The AMCC S593X header file                  */
/*                                                                */
/******************************************************************/
/*                                                                */
/* File Name   : S593X.h                                          */
/* Title       : The AMCC S593X header file                       */
/* Description : Defines the registers of the S593X chip          */
/* Authors     : Frederic Pennerath for the S-LINK specific       */
/*               functions                                        */
/*               David Francis for the AMCC and PCI functions     */
/* Revisions   :                                                  */
/*               E. Denes (25.05.01) change of mbef definition    */
/*                                                                */
/******************************************************************/

/******************************************************************/
/*                                                                */
/*               Registers' structure definition                  */
/*                                                                */
/******************************************************************/

typedef union {
  char           byte[4];
  unsigned       all;
} mailbox;

typedef struct {
  unsigned char  b0  : 1;
  unsigned char  b1  : 1;
  unsigned char  b2  : 1;
  unsigned char  b3  : 1;
  unsigned char  b4  : 1;
  unsigned char  b5  : 1;
  unsigned char  b6  : 1;
  unsigned char  b7  : 1;
  unsigned char  b8  : 1;
  unsigned char  b9  : 1;
  unsigned char  b10 : 1;
  unsigned char  b11 : 1;
  unsigned char  b12 : 1;
  unsigned char  b13 : 1;
  unsigned char  b14 : 1;
  unsigned char  b15 : 1;
} mbefbits;

typedef union {
    mbefbits       bit;
    unsigned short all;
  } mbef_out;

typedef union {
    mbefbits       bit;
    unsigned short all;
  } mbef_in;

typedef union {
    struct {
        mbef_out out;
        mbef_in in;
    } part;
    unsigned all;
} mbef;

typedef struct {
  unsigned char     omi     : 1;
  unsigned char     imi     : 1;
  unsigned char     wtc     : 1;
  unsigned char     rtc     : 1;
  unsigned char     ma      : 1;
  unsigned char     ta      : 1;
  unsigned char             : 1;
  unsigned char     ia      : 1;

} intstatus;

typedef struct {
  unsigned char     ec     : 2;
  unsigned char     facp   : 2;
  unsigned char     faca   : 2;
  unsigned char     inf    : 1;
  unsigned char     ouf    : 1;
} intendian;

typedef struct {
  unsigned char     outbyte : 2;
  unsigned char     outmbox : 2;
  unsigned char     outien  : 1;
  unsigned char             : 3;

  unsigned char     inbyte  : 2;
  unsigned char     inmbox  : 2;
  unsigned char     inien   : 1;
  unsigned char             : 1;
  unsigned char     wtc     : 1;
  unsigned char     rtc     : 1;
} intselect;

typedef union {
  struct {
    intselect   select;
    intstatus   status;
    intendian   endian;
  } part;
  unsigned all;
} intcsr;

typedef struct {
  unsigned char  p2aff   : 1;
  unsigned char  p2af4   : 1;
  unsigned char  p2afe   : 1;
  unsigned char  a2pff   : 1;
  unsigned char  a2pf4   : 1;
  unsigned char  a2pfe   : 1;
  unsigned char  p2atc   : 1;
  unsigned char  a2ptc   : 1;

  unsigned char  wvr     : 1;
  unsigned char  wms     : 1;
  unsigned char  wte     : 1;
  unsigned char          : 1;
  unsigned char  rvw     : 1;
  unsigned char  rms     : 1;
  unsigned char  rte     : 1;
  unsigned char          : 1;

  unsigned char  nvop;

  unsigned char  aor     : 1;
  unsigned char  p2afr   : 1;
  unsigned char  a2pfr   : 1;
  unsigned char  mfr     : 1;
  unsigned char          : 1;
  unsigned char  nvctl   : 3;
} mcsrbits;

typedef union {
  mcsrbits       bit;
  unsigned       all;
} mcsr;

typedef struct {
  mailbox      omb[4];           /* Outgoing Mailbox Register 1          */
  mailbox      imb[4];           /* Incoming Mailbox Register 1          */
  unsigned     fifo;             /* Bidirectional FIFO Port              */
  unsigned     mwar;             /* Master Write Address Register        */
  unsigned     mwtc;             /* Master Write Transfer Count Register */
  unsigned     mrar;             /* Master Read Address Register         */
  unsigned     mrtc;             /* Master Read Transfer Count Register  */
  mbef         mbef;             /* Mailbox Empty/Full Status            */
  intcsr       intcsr;           /* Interrupt Control/Status Register    */
  mcsr         mcsr;             /* Bus Master Control/Status Register   */
} s593x_regs;

