/********************************************************
*   ddl_init_link.c                                     *
*   general link setup program with master              *
*   and slave options                                   *
*   last update:        07/08/2004                      *
*   written by: Peter Csato and Ervin Denes             *
********************************************************/
#define SOURCE_FILE "ddl_init_link.c"
#define DESCRIPTION "DDL link setup program"

#include <ctype.h>
#include <signal.h>
#include "rorc_ddl.h"

int minor = 0;
int channel = 0;
int master = 1;
int print = -1;
int reset = 1;
int stop = 1;

static struct sigaction sigac;
rorc_pci_dev_t prorc;

/*********************************************************/
void print_usage(char* s)
{
   printf("Usage: %s [-{M|m} <minor>] [-{C|c} <channel>] [-{P|p} <print>]\n",s);
   printf("                     [-{S|s}] [-{T|t}] [-{N|n}]\n");
   printf("-M minor    : RORC number (default: 0)\n");
   printf("-C channel  : RORC's channel number (default: 0)\n");
   printf("-p print    : # of status prints (default: -1 means no limit)\n");
   printf("-s          : slave (default: master)\n");
   printf("-t          : test: no stop (default: stop if link is active)\n");
   printf("-n          : no reset (default: reset)\n");
}


/********************************************************
*          signal handler                               *
********************************************************/
void sig_hand(int signum)
{
/* signum: 1(SIGHUP)  2(SIGINT)  3(SIGQUIT) */
  if(signum < 4)
  {
    /* close RORC */
    rorcClose(&prorc);
    exit (0);
  }
}

/********************************************************
*          signal handling                              *
********************************************************/
void set_sig_hand(void)
{
  sigac.sa_flags = 0;
  sigac.sa_handler =  sig_hand;
  if (sigaction(SIGHUP, &sigac, NULL) < 0 )
  {
     perror(" sigaction SIGHUP ");
     exit(1);
  }
  if (sigaction(SIGINT, &sigac, NULL) < 0 )
  {
     perror(" sigaction SIGINT ");
     exit(1);
  }
  if (sigaction(SIGQUIT, &sigac, NULL) < 0 )
  {
     perror(" sigaction SIGQUIT ");
     exit(1);
  }
}


/*************************************************************/
int main(int argc, char **argv)
{

  unsigned long retlong;
  int c, n;
  int TimeOut, usTimeOut = DDL_RESPONSE_TIME;

  /* process arguments */
  if(argc  > 1 ) 
    while ((c = getopt(argc, argv, "M:m:C:c:P:p:HhSsTtNn")) != EOF) 
    {
       switch (tolower(c)) 
       {
          case 'm': minor = atoi(optarg); break;
          case 'c': channel = atoi(optarg);
            if ((channel < 0) || (channel >= MAX_CHANNEL))
            {
              printf(" Channel number should be positive and less than %d\n",
                                  MAX_CHANNEL);
              print_usage (argv[0]);
              exit (EXIT_FAILURE);
             }
            break;
          case 'p': print = atoi(optarg); break;
          case 's': master = 0; break;
          case 't': stop = 0; break;
          case 'n': reset = 0; break;
          default : print_usage(argv[0]); exit(0); break;
       }
    }

  /* open RORC and map mailboxes */
  if (rorcOpenChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);
  TimeOut = usTimeOut * prorc.loop_per_usec;

  /* Initialization of the RORC */
  if(reset) 
  {
     rorcReset(&prorc, RORC_RESET_DIU);
     rorcReset(&prorc, 0); // reset RORC
     printf(" RORC initialized\n");
  }

  /* clear RORC's FIFOs (but FF) */
  rorcReset(&prorc, RORC_RESET_FIFOS);

  /* empty incoming Status FIFO if necessary */
  if (pRorc(&prorc))
    n = pRorcEmptyMb(&prorc, 1, 0);
  else
    n = dRorcEmptyRxStatus(&prorc, 0);
  if (n)
    printf ("Before sending DDL command, DDL status register contained %d status.\n", n);

  /* signal handling */
  set_sig_hand();

  /* init link */
  retlong = ddlLinkUp(&prorc, master, print, stop, TimeOut);
  if (retlong == -1)
  {
    printf(" Can not read DIU status\n");
  }

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);
}
