/***************************************************************
* drorc_empty_ddlst.c                                          *
* empties D-RORC RX status FIFO        i                       *
* Written by E. Denes                                          *
* Last modified on 22/08/2003                                  *
****************************************************************/
#define SOURCE_FILE "drorc_empty_ddlst.c"
#define DESCRIPTION "Emptying D-RORC RX Status FIFO"    

#include <ctype.h>
#include "rorc_ddl.h"

/*************************************************************/
void usage(char *s)
{
   printf("\n-----------------------------------------------------------\n");
   printf("Usage: %s [-{M|m} <minor>] [-{C|c} <channel>\n", s);
   printf("   where\n");
   printf("   <minor>          RORC card number. Default = 0.\n");
   printf("   <channel>        RORC's channel number. Default = 0.\n");
   printf("-----------------------------------------------------------\n\n");
}

/****************************************************************************/

int main (int argc, char **argv)
{

  int c, minor, channel;
  rorc_pci_dev_t prorc;

  /* treat parameters */

  minor = 0;
  channel = 0;

  if(argc > 1)
    while((c = getopt(argc, argv, "M:m:C:c:")) != EOF)
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); break;
                case 'C': channel = atoi(optarg);
          if ((channel < 0) || (channel >= MAX_CHANNEL))
          {
            printf(" Channel number should be positive and less than %d\n",
                                MAX_CHANNEL);
            usage (argv[0]);
            exit (EXIT_FAILURE);
          }
          break;
        default : usage(argv[0]); exit (EXIT_FAILURE); break;
      }

  /* open RORC and map registers */
  if (rorcMapChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);

  if (pRorc(&prorc))
  {
    printf("This program works only for RORC revision greater than 1\n\n");
    rorcClose(&prorc);
    exit (EXIT_FAILURE);
  }

  /* Read and print the received status */
  dRorcEmptyRxStatus(&prorc, 1);

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);

}
