/****************************************************
 * drorc_wr_reg
 *
 * Accessing D-RORC register area
 *
 * last updated: 20/01/2004
 * written by: Ervin Denes
 ****************************************************/
#define SOURCE_FILE "drorc_wr_reg.c"
#define DESCRIPTION "Accessing D-RORC register area"

#include <ctype.h>
#include "rorc_ddl.h"

#define ADDR_MAX 0xFC

/******************************************************/
void usage(char *name)
{
  printf("usage: %s [-{M|m} <minor>|0] [-{C|c} <channel>|0]\n", name);
  printf("                   [-{A|a} <register_addres|0>] [-{V|v} <hex value>]\n");
  printf("       Maximum register address value is 0x%x\n", ADDR_MAX);
  printf("       If data value not given, the given register will be read out.\n");
  printf("       The program works only if RORC has revision number > 1.\n");
}
	
/******************************************************/
int main(int argc, char **argv)
{
  int c, minor, channel;
  int addr_max, no_val;

  rorc_pci_dev_t prorc;
  volatile unsigned long *reg_addr;
  unsigned long reg_val;
  int addr = 0;
  __u32 data = 0;

  no_val = 1;
  addr_max = ADDR_MAX;

  /* Check parameters */
  minor = 0;
  channel = 0;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:C:c:A:a:V:v:")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); if (minor < 0) minor = 0; break;
        case 'C':
          channel = strtoul(optarg, (char**)NULL, 16); 
          if ((channel < 0) || (channel >= MAX_CHANNEL))
          {
            printf(" Channel number should be positive and less than %d\n",
                                       MAX_CHANNEL);
            usage(argv[0]);
            exit (EXIT_FAILURE); 
          }
          break;
        case 'A':
          addr = strtoul(optarg, (char**)NULL, 16); 
          if (addr > addr_max)
          {
            printf(" Register address value should be >= 0x%x\n", ADDR_MAX);
            usage(argv[0]);
            exit (EXIT_FAILURE); 
          }
          break;
        case 'V':
          data  = strtoul(optarg, (char**)NULL, 16); no_val = 0; break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }

  if (rorcMapChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);

  if (pRorc(&prorc))
  {
    printf(" The program works only if RORC has revision number > 1.\n");
    goto vege;
  }

  reg_addr = prorc.reg[0] + (addr >> 2);
  if (!no_val)
    *reg_addr = data;

  reg_val = *reg_addr;
  printf(" %8p: 0x%08lx\n", reg_addr, reg_val);

vege:
  rorcClose(&prorc);
  return 0;
}

