/***************************************************************
* prorc_empty_mb.c                                             *
* read and print an incoming mailbox of a pRORC                *
* Written by E. Denes                                          *
* Last modified on 22/08/2003                                  *
****************************************************************/
#define SOURCE_FILE "prorc_empty_mb.c"
#define DESCRIPTION "Emptying pRORC incoming mailboxes"

#include <ctype.h>
#include "rorc_ddl.h"

/*************************************************************/
void usage(char *s)
{
   printf("\n-----------------------------------------------------------\n");
   printf("Usage: %s [-{M|m} minor] [-{N|n} mailbox_number]\n", s);
   printf("   where\n");
   printf("   minor          pRORC number. Default = 0.\n");
   printf("   mailbox_number number of the incoming mailbox to empty.\n");
   printf("                  Possible values: [1-4], default = 1.\n");
   printf("-----------------------------------------------------------\n\n");
}

/****************************************************************************/

int main (int argc, char **argv)
{

  int c, minor, mb_number;
  rorc_pci_dev_t prorc;

  /* treat parameters */

  minor = 0;
  mb_number = 1;

  if(argc > 1)
    while((c = getopt(argc, argv, "M:m:N:n:")) != EOF)
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); break;
        case 'N':
          mb_number = atoi(optarg);
          if(mb_number < 1 || mb_number > 4)
          {
             printf("Invalid mail box number!\n");
             usage(argv[0]);
             exit (EXIT_FAILURE);
          }
          break;
        default: usage(argv[0]); exit (EXIT_FAILURE); break;
      }

  /* open RORC and map mailboxes */
  if (rorcMap(&prorc, minor))
    exit (EXIT_FAILURE);

  if (!pRorc(&prorc))
  {
    printf("This program works only for RORC revision 1\n\n");
    rorcClose(&prorc);
    exit (EXIT_FAILURE);
  }

  /* Read and print the mb */
  pRorcEmptyMb(&prorc, mb_number, 1);

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);

}
