/****************************************************
 * prorc_mbread.c
 *
 *  Accessing S593x incoming / outgoing  mailboxes 
 *  using mmap() on /dev/prorc0
 *
 * last updated: 17/04/2003
 * written by: Peter Csato and Ervin Denes
 ****************************************************/
#define SOURCE_FILE "prorc_mbread.c"
#define DESCRIPTION "Read S593x mailboxes"

#include <ctype.h>
#include "rorc_ddl.h"

/*******************************************************/
void usage(char *name)
{
  printf("Reads S593x incoming and outgoing mailboxes.\n");
  printf("usage: %s [-{M|m} <pRORC minor number>|0]\n", name);
  printf("       Useable only for ALICE RORC revision 1 (AMCC)\n");
}
/*******************************************************/

int main(int argc, char **argv)
{
  int c, i, minor; 

  rorc_pci_dev_t prorc;
  int mb_min = 1, mb_max = 4;

  /* Check parameters */
  minor = 0;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); if (minor < 0) minor = 0; break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }

  if (rorcMap(&prorc, minor))
    return -1;

  if (!pRorc(&prorc))
  {
    usage (argv[0]); 
    exit (EXIT_FAILURE);
  }

  printf("MBEF: %08lx\n", *prorc.mbef);

  printf("\t\t");
  for (i = mb_min; i <= mb_max; i++)
    printf("OMB%d: %08lx\t", i, *prorc.omb[i]);
  printf("\n");

  printf("\t\t");
  for (i = mb_min; i <= mb_max; i++)
    printf("IMB%d: %08lx\t", i, *prorc.imb[i]);
  printf("\n");

  printf("MBEF: %08lx\n", *prorc.mbef);
  
  rorcClose(&prorc);

  return 0;
}

