/****************************************************
 * prorc_mbreset.c
 *
 *  Resetting S593x incoming / outgoing  mailboxes 
 *  using ioctl() on /dev/prorc0
 *
 * last updated: 17/04/2003
 * written by: Peter Csato and Ervin Denes
 ****************************************************/
#define SOURCE_FILE "prorc_mbreset.c"
#define DESCRIPTION "Reset S593x mailboxes"

#include <ctype.h>
#include "rorc_ddl.h"

/*******************************************************/
void usage(char *name)
{
  printf("Resets S593x incoming and outgoing  mailboxes.\n");
  printf("usage: %s [-{M|m} <pRORC minor number>|0]\n", name);
  printf("       Useable only for ALICE RORC revision 1 (AMCC)\n");
}

/*******************************************************/
int main(int argc, char **argv)
{
  int c, minor, fd;
  unsigned long ret;
  rorc_pci_dev_t prorc;

  /* Check parameters */
  minor = 0;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); if (minor < 0) minor = 0; break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }

  if (rorcMap(&prorc, minor))
    return -1;

  fd = prorc.fd;

  if (!pRorc(&prorc))
  {
    usage (argv[0]);
    exit (EXIT_FAILURE);
  }

  printf("MBEF: %08lx\n", *prorc.mbef);

  if(ioctl(fd, PRORC_MB_RESET, &ret) == -1)
    perror("ioctl");
  printf("\t\tMailbox flags reset\n");

  printf("MBEF: %08lx\n", *prorc.mbef);

  return 0;
}

