/****************************************************
 * prorc_mbwrite.c
 *
 *  Accessing S593x incoming / outgoing  mailboxes 
 *  using mmap() on /dev/prorc0
 *
 * last updated: 17/04/2003
 * written by: Peter Csato and Ervin Denes
 ****************************************************/
#define SOURCE_FILE "prorc_mbwrite.c"
#define DESCRIPTION "Write S593x mailboxes"

#include <ctype.h>
#include "rorc_ddl.h"

/******************************************************/
void usage(char *name)
{
  printf("usage: %s [-{M|m} <minor>|0] -{N|n} <mb#[1-4]> -{V|v} <hex value>\n", 
                   name);
  printf("       Useable only for ALICE RORC revision 1 (AMCC)\n");
}

/******************************************************/
int main(int argc, char **argv)
{
  int c, i, minor;
  int no_mb, no_val;

  rorc_pci_dev_t prorc;
  int mb, mb_min = 1, mb_max = 4;
  __u32 data;

  mb = 0;
  no_mb = 1;
  data = 0;
  no_val = 1;

  /* Check parameters */
  minor = 0;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:N:n:V:v:")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); if (minor < 0) minor = 0; break;
        case 'N':
          mb = atoi(optarg);
          if ((mb < 1) || (mb > 4))
          {
            usage(argv[0]);
            exit (EXIT_FAILURE); 
          }
          no_mb = 0;
          break;
        case 'V':
          data  = strtoul(optarg, (char**)NULL, 16); no_val = 0; break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }
  if(no_mb || no_val)
  {
    usage(argv[0]);
    exit (EXIT_FAILURE); 
  }

  if (rorcMap(&prorc, minor))
    exit (EXIT_FAILURE);

  if (!pRorc(&prorc))
  {
    usage (argv[0]);
    exit (EXIT_FAILURE);
  }

  printf("MBEF: %08lx\n", *prorc.mbef);

  *prorc.omb[mb] = data;

  printf("\t\t");
  for (i = mb_min = 1; i <= mb_max; i++)
    printf("OMB%d: %08lx\t", i, *prorc.omb[i]);
  printf("\n");

  printf("MBEF: %08lx\n", *prorc.mbef);

  rorcClose(&prorc);

  return 0;
}

