/*********************************
*  Mini Web Server for the pRORC *
*  12.09.2001.                   *
*********************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <assert.h>

#include <unistd.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <sys/socket.h>
#include <sys/time.h>
#include <fcntl.h>
#include <signal.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include "prorc_ws.h"
#include "rorc_lib.h"

#define MAX_HEAD_SIZE           1024
#define MAX_BODY_SIZE           50000
#define MAX_REQUEST_SIZE        MAX_HEAD_SIZE
#define MAX_RESPONSE_SIZE       (MAX_HEAD_SIZE + MAX_BODY_SIZE)
#define MAX_PENDING_CONN        3
#define PORT_NUM                8001
#define MAX_LINE		80
#define MAXINLINE		256

typedef unsigned char byte;

static int errcheck= TRUE;
static char *version="0.2";
static int prorcs=0,pcount;

static int      sockfd;              /* server socket */
char *log_file = "../../tmp/prorc_ws.log";
FILE *file;
time_t t;
char *method, *path, *protocol, *agent, *host, *auth;
char *decoration="<style type=\"text/css\">
  A:link {text-decoration: none; color: green}
  A:visited {text-decoration: none; color: green}
  A:active {text-decoration: none}
  A:hover {text-decoration: underline; color: red;}
  TR {text-align: center;}
  </style>";

char *secret="Authorization: Basic";

/* HTTP buffers */
static char     head_buff[MAX_HEAD_SIZE + 1];
static char     request_buff[MAX_REQUEST_SIZE + 1];
static char     body_buff[MAX_BODY_SIZE + 1];

static char *vendor_id="CERN/ECP/EDU";

static  char *request_header[19]={
  "Accept","Accept-Charset","Accept-Encoding","Accept-Language",
  "Authorization","Expect","From","Host","If-Match","If-Modified-Since",
  "If-None-Match","If-Rnage","If-Unmodified-Since","Max-Forwards",
  "Proxy-Authorization","Range","Referer","TE","User-Agent"};

static char command[256]="sr 2<&1";


/****************************
*       functions           *
****************************/

/****************************************************/
decode(char *iobuf, char *out)
{
  static byte dtable[256];
  int i,c,j=1,count=0,count2=0;

  for(i= 0;i<255;i++){
    dtable[i]= 0x80;
  }
  for(i= 'A';i<='I';i++){
    dtable[i]= 0+(i-'A');
  }
  for(i= 'J';i<='R';i++){
    dtable[i]= 9+(i-'J');
  }
  for(i= 'S';i<='Z';i++){
    dtable[i]= 18+(i-'S');
  }
  for(i= 'a';i<='i';i++){
    dtable[i]= 26+(i-'a');
  }
  for(i= 'j';i<='r';i++){
    dtable[i]= 35+(i-'j');
  }
  for(i= 's';i<='z';i++){
    dtable[i]= 44+(i-'s');
  }
  for(i= '0';i<='9';i++){
    dtable[i]= 52+(i-'0');
  }
  dtable['+']= 62;
  dtable['/']= 63;
  dtable['=']= 0;

  while(count<strlen(iobuf)){
    byte a[4],b[4],o[3];

    for(i= 0;i<4;i++){
      c = iobuf[count++];

      if(dtable[c]&0x80){
        if(errcheck){
          fprintf(stderr,"Illegal character: '%c'.\n",c);
        }

        i--;
        continue;
      }
      a[i]= (byte)c;
      b[i]= (byte)dtable[c];
    }
    o[0]= (b[0]<<2)|(b[1]>>4);
    o[1]= (b[1]<<4)|(b[2]>>2);
    o[2]= (b[2]<<6)|b[3];
    i= a[2]=='='?1:(a[3]=='='?2:3);
    out[count2] = o[0];
    out[count2+1] = o[1];
    out[count2+2] = o[2];
    count2+=3;
  }
  out[count2++]= EOS;
}

/****************************************************/
int create_img_html(int status_code, const char *status_desc, const char *title, int sd, char *fname,int length)
{
 FILE *fd;
  int i;
  char *in;

  fd=fopen(fname,"r");
  if(fd<0) return -1;
  i=fread(body_buff,1,length,fd);
  if(i!=length)
    return -1;

  snprintf(head_buff, MAX_HEAD_SIZE - 1,
    "HTTP/1.0 %d %s %s\r\n"
    "Server: pRORC 0.1\r\n"
    "Connection: close\r\n"
    "Content-Type: image/jpg\r\n"
    "Content-Lenght: %ld\r\n\r\n",
    status_code, status_desc, title, length);


  if(write(sd,head_buff,strlen(head_buff)) != -1)
  {
    if(write(sd,body_buff,length) == -1)
      error("http_send_response: %s\n", strerror(errno));
  } else {
    error("http_send_response: %s\n", strerror(errno));
  }

  return 0;
}




/****************************************************/
void create_index_html()
{

  snprintf(body_buff, MAX_BODY_SIZE,
    "<HTML>\n"
    "<HEAD><TITLE>pRORC Mini Web Server</TITLE> %s"
    "</HEAD>\n"
    "<BODY BGCOLOR=\"WHITE\"><CENTER><P>\n"
    "<H2>pRORC Mini Web Server</H2>\n"
    "<TABLE>\n"
    "<TR><TD><A HREF=\"/info.html\">PCI Info</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/tests.html\">pRORC Tests</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/help.html\">About</A></TD></TR>\n"
    "</TABLE><P>\n"
    "</CENTER>\n"
    "<CENTER><IMG SRC=\"prorc3.jpg\"></CENTER>\n"
    "<br><div align=right><b>%d (%d)</b> pRORC card(s) found.</div><br>\n"
    "<HR><ADDRESS>pRORC Mini Web Server %s<ADDRESS>\n"
    "</BODY>\n"
    "</HTML>\n",decoration,prorcs,pcount,version);
}

/****************************************************/
void create_help_html()
{
  snprintf(body_buff, MAX_BODY_SIZE,
    "<HTML>\n"
    "<HEAD><TITLE>pRORC Info Page</TITLE> %s"
    "</HEAD>\n"
    "<BODY BGCOLOR=WHITE><CENTER><P>\n"
    "<H2>pRORC Info Page</H2>\n"
    "pRORC is a PCI based Read-out Receiver Card developed for the ALICE DDL project<BR>by KFKI-RMKI Budapest, Technical University Budapest and CERN.<BR><BR>\n"
    "<CENTER><IMG SRC=\"/prorc1.jpg\"></CENTER>\n"
    "<BR><A HREF=\"/\">Main Page</A></TD></TR>\n"
    "</CENTER>\n"
    "<HR><ADDRESS>pRORC Mini Web Server %s<ADDRESS>\n"
    "</BODY>\n"
    "</HTML>\n",decoration,version);
}



/****************************************************/
void create_tests_html()
{
  char radiobu[300], *radio;
  char *javas="<script language=\"javascript\">
    function  prorc(ref)
    {
      if(document.forms[0].elements.length > 1)
      {
        for (i=0; i<document.forms[0].elements.length; i++ ) {
          if (document.forms[0].choice[i].checked==\"1\") {
            ref+='?prorc='+i;
          } 
        }
      } else {
        ref+='?prorc=0';
      }
      return ref;
    }
    </script>";

  int i;

  radio = radiobu;
  for(i=0;i<prorcs;i++)
  {
    if(i==0)
    {
      sprintf(radio,"<INPUT TYPE=\"radio\" NAME=\"choice\" VALUE=\"%d\" CHECKED> pRORC%d  &nbsp;&nbsp;",i,i);
      radio+=73;
    } else {
      sprintf(radio,"<INPUT TYPE=\"radio\" NAME=\"choice\" VALUE=\"%d\"> pRORC%d  &nbsp;&nbsp;",i,i);
    }
  }

  snprintf(body_buff, MAX_BODY_SIZE,
    "<HTML>\n"
    "<HEAD><TITLE>pRORC Tests Page</TITLE> %s\n"
    "%s</HEAD>\n"
    "<BODY BGCOLOR=WHITE><CENTER><P>\n"
    "<H2>pRORC Tests Page</H2>\n"
    "<BR>Please select a pRORC card:<BR>\n"
    "<BR><FORM>%s\n"
    "</FORM>\n"
    "<TABLE>\n"
    "<TR><TD><A HREF=\"/sr.html\" onclick=\"this.href=prorc('sr.html')\"\">Show pRORC Registers</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/prid.html\" onclick=\"this.href=prorc('prid.html')\"\">Show pRORC, DIU and SIU firmware id</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/preset.html\" onclick=\"this.href=prorc('preset.html')\"\">pRORC Reset</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/emptymb.html\" onclick=\"this.href=prorc('emptymb.html')\"\">Empty pRORC mailboxes</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/ss.html\" onclick=\"this.href=prorc('ss.html')\"\">Show pRORC Status</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/diustat.html\" onclick=\"this.href=prorc('diustat.html')\"\">Show DIU Status</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/siustat.html\" onclick=\"this.href=prorc('siustat.html')\"\">Show SIU Status</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/feicstat.html\" onclick=\"this.href=prorc('feicstat.html')\"\">Show Front-end Emulator Status</A></TD></TR>\n"
    "<TR><TD><A HREF=\"/\">Main Page</A></TD></TR>\n"
    "</TABLE><P>\n"
    "</CENTER>\n"
    "<HR><ADDRESS>pRORC Mini Web Server %s<ADDRESS>\n"
    "</BODY>\n"
    "</HTML>\n",decoration,javas,radiobu,version);
}

/****************************************************/
static int search_string(char *s1, char *s2)
{
  int i;

  for(i=0;i<strlen(s2);i++)
  {
     if(bcmp(s2+i,s1,strlen(s1)) == 0)
       return(1);
  }
  return(0);
}

/****************************************************/
static char *prorc_url(char *s1)
{
  int i;
  char *param,*par0="0";

  param=par0;
  for(i=0;i<strlen(s1);i++)
  {
     if(bcmp(s1+i,"?prorc=",7) == 0)
     {
       param=s1+i+7;
     }
  }
  return(param);
}



/****************************************************/
get_prorc_info()
{
  FILE *pp;
  #define BUFF_SIZE	1024
  char command_buffer[BUFF_SIZE*MAX_LINE];
  char *vbuff,tempbuff[BUFF_SIZE];
  int i=0;

  if ((pp = popen( "/sbin/lspci 2<&1","r")) == NULL)
  {
    printf("Failed to open pipe\n");
    exit(-1);
  }
 
  vbuff=command_buffer;
  while (!feof(pp) && (i<MAX_LINE))
  {
    if(fgets(tempbuff,BUFF_SIZE,pp) != NULL)
    {
      if(search_string(vendor_id,tempbuff))
        prorcs++;
    i++;
    }
  }
  pclose(pp);
}

/****************************************************/
void create_command_html(char *command_name)
{
FILE *pp;
#define BUFF_SIZE       1024
char command_buffer[BUFF_SIZE*MAX_LINE];
char *vbuff;
int i=0;

if ((pp = popen(command,"r")) == NULL)
   {
   printf("Failed to open pipe\n");
   exit(-1);
   }

vbuff=command_buffer;
while (!feof(pp) && (i<MAX_LINE))
{
  if(fgets(vbuff,BUFF_SIZE,pp)!= NULL)
  {
    vbuff+=strlen(vbuff);
    i++;
  }
} 

pclose(pp);

snprintf(body_buff, MAX_BODY_SIZE,
  "<HTML>\n"
  "<HEAD><TITLE>pRORC PCI Info Page</TITLE> %s"
  "</HEAD>\n"
  "<BODY BGCOLOR=\"WHITE\"><CENTER><P>\n"
  "<H2>%s</H2>(for pRORC %s)\n"
  "<DIV align=\"left\">"
  "<PRE>%s</PRE>"
  "</DIV>"
  "<TABLE>\n"
  "<TR align=center><TD><A HREF=\"/tests.html\">pRORC Tests Page</A></TD></TR>\n"
  "<TR align=center><TD><A HREF=\"/\">Main Page</A></TD></TR>\n"
  "</TABLE><P>\n"
  "</CENTER>\n"
  "<HR><ADDRESS>pRORC Mini Web Server %s<ADDRESS>\n"
  "</BODY>\n"
  "</HTML>\n",decoration,command_name,prorc_url(path),command_buffer,version);
}

/****************************************************/
void create_info_html()
{
FILE *pp;
#define BUFF_SIZE	1024
char command_buffer[BUFF_SIZE*MAX_LINE];
char fcolorb[21]="<font color=\"red\">"; 
char fcolore[8]="</font>"; 
char *vbuff,tempbuff[BUFF_SIZE];
int i=0;

if ((pp = popen( "/sbin/lspci 2<&1","r")) == NULL)
   {
   printf("Failed to open pipe\n");
   exit(-1);
   }
 
vbuff=command_buffer;
while (!feof(pp) && (i<MAX_LINE))
{
  if(fgets(tempbuff,BUFF_SIZE,pp) != NULL)
  {
    if(search_string(vendor_id,tempbuff))
    {
      snprintf(vbuff,strlen(fcolorb)+1,"%s",fcolorb);
      vbuff+=strlen(fcolorb);
      memcpy(vbuff,tempbuff,strlen(tempbuff));
      vbuff+=strlen(tempbuff);
      snprintf(vbuff,strlen(fcolore)+1,"%s",fcolore);
      vbuff+=strlen(fcolore);
    } else {
      memcpy(vbuff,tempbuff,strlen(tempbuff));
      vbuff+=strlen(tempbuff);
    }
    i++;
  }
}
 
pclose(pp);

snprintf(body_buff, MAX_BODY_SIZE,
  "<HTML>\n"
  "<HEAD><TITLE>pRORC PCI Info Page</TITLE> %s"
  "</HEAD>\n"
  "<BODY BGCOLOR=\"WHITE\"><CENTER><P>\n"
  "<H2>PCI Devices:</H2>\n"
  "<DIV align=\"left\">"
  "<PRE>%s</PRE>"
  "</DIV>"
  "<TABLE>\n"
  "<TR><TD><A HREF=\"/\">Main Page</A></TD></TR>\n"
  "</TABLE><P>\n"
  "</CENTER>\n"
  "<HR><ADDRESS>pRORC Mini Web Server %s<ADDRESS>\n"
  "</BODY>\n"
  "</HTML>\n",decoration,command_buffer,version);
}


/****************************************************/
void create_socket()
{
  struct sockaddr_in serv_addr;
  int true = TRUE;

  sockfd = socket(AF_INET, SOCK_STREAM, 0);
  if (sockfd < 0)
        error("ERROR opening socket");

  setsockopt(sockfd,SOL_SOCKET,SO_REUSEADDR,&true,sizeof(true));

  bzero((char *) &serv_addr, sizeof(serv_addr));
  serv_addr.sin_family = AF_INET;
  serv_addr.sin_addr.s_addr = INADDR_ANY;
  serv_addr.sin_port = htons(PORT_NUM);

  if (bind(sockfd, (struct sockaddr *) &serv_addr, sizeof(serv_addr)) < 0)
    error("ERROR on binding");
  listen(sockfd,5);
}

/****************************************************/
void http_create_response_head(int status_code, const char *status_desc,
  const char *title, long size, const char *mime_type)
{
  snprintf(head_buff, MAX_HEAD_SIZE - 1,
    "HTTP/1.0 %d %s %s\r\n"
    "Server: pRORC 0.1\r\n"
    "Connection: close\r\n"
    "Content-Type: %s\r\n"
    "Content-Lenght: %ld\r\n\r\n",
    status_code, status_desc, title, mime_type, size);
}

/****************************************************/
void http_create_authresponse_head(int status_code, const char *status_desc,
  const char *title, long size, const char *mime_type)
{
  snprintf(head_buff, MAX_HEAD_SIZE - 1,
    "HTTP/1.0 %d %s %s\r\n"
    "Server: pRORC %s\r\n"
    "Connection: close\r\n"
    "Content-Type: %s\r\n"
    "WWW-Authenticate: Basic realm=\"pRORC\"\r\n"
    "Content-Lenght: %ld\r\n\r\n",
    status_code, status_desc, version, title, mime_type, size);
}


/****************************************************/
void http_send_response(int sd)
{
  int n;

  if(write(sd,head_buff,strlen(head_buff)) != -1)
  {
    if(write(sd,body_buff,strlen(body_buff)) == -1)
      error("http_send_response: %s\n", strerror(errno));
  } else {
    error("http_send_response: %s\n", strerror(errno));
  }
}

/****************************************************/
void http_send_error(int status_code, int sd)
{
  const char *text;
  static const char *body = "<HTML><HEAD>\n"
    "<TITLE>%d %s</TITLE>\n"
    "</HEAD><BODY BGCOLOR=\"WHITE\"><BR>\n"
    "<H2>HTTP %d</H2>\n"
    "%s\n"
    "<P><HR>\n"
    "<ADDRESS>pRORC %s</ADDRESS>\n"
    "</BODY></HTML>\n";

  switch (status_code) 
  {
    case 400:	text = "Bad request";                   break;
    case 401:	text = "Unauthorized";                  break;
    case 403:	text = "Forbidden";                     break;
    case 404:	text = "Not found";                     break;
    case 405:	text = "Method not allowed";            break;
    case 408:	text = "Request timeout";               break;
    case 413:	text = "Request entity too large";      break;
    case 414:	text = "Request URI too large";         break;
    case 500:	text = "Internal server error";         break;
    case 501:	text = "Not implemented";               break;
    case 503:	text = "Service unaviable";             break;
    default:	error("Not used HTTP error code %d\n", status_code);
                text = "Internal server error";
                status_code = 500;
  }

  snprintf(body_buff, MAX_BODY_SIZE - 1, body, status_code, text,
    status_code, text, version);

  if(status_code != 401)
  {
    http_create_response_head(status_code, text, "HTTP error",
      strlen(body_buff), "text/html");
  } else
  {
    http_create_authresponse_head(status_code, text, "HTTP error",
      strlen(body_buff), "text/html");
  }

  http_send_response(sd);
}


/****************************************************/
int http_parse_request()
{
  int i=0,j=0,k,ch;
  char *step, *pass;
  char outbuf[MAXINLINE];
 
  do{
    ch = request_buff[i];
    if(ch  == '\r' || ch == '\n')
    {     
      request_buff[i] = EOS;
      j++;
    }
    i++;
  }while(j<1);

  step=agent=auth=request_buff+i;
  j=0;

  do{
    ch = step[0];
    if(ch == '\n')
    {     
     step[0] = EOS;
     step++;
      if(!bcmp(step,request_header[18],strlen(request_header[18]))) 
      {
        j++;
        agent = step;
      } 
      if(!bcmp(step,request_header[4],strlen(request_header[4])))
      {
        j++;
        auth = step;
      }
    }
    if(ch == EOS)
      j=3;
    step++;
  }while(j<3);

  method=request_buff;
  j=i=0; 
  do{
   ch = request_buff[i]; 
   if(ch  == ' ')
   {
     *(method+i) = EOS;
     j++;
   }
   i++;
  }while(j<1);
  
  path=request_buff+i;
    j=i=0;
  do{
   ch = path[i];
   if(ch  == ' ')
   {
     *(path+i) = EOS;
     j++;
   }
   i++;
  }while(j<1);

  protocol=path+i;
    j=i=0;
  do{
   ch = protocol[i];
   if(ch  == ' ')
   {
     *(protocol+i) = EOS;
     j++;
   }
   i++;
  }while(j<1);

  time(&t);
  fprintf(file,"%s %s %s %s \n %s\n",ctime(&t),method, path, protocol,agent); 
  fprintf(file," Auth: %s\n",auth);
  fflush(file);

  if(!search_string("GET", method))
    return -2;

  if(search_string(secret, auth))
  {
    j=0;
    k=strlen(auth);
    for(i=0;i<k;i++)
    {
     ch = auth[i];
     if(ch  == ' ') 
     {
       j++;
       auth[i] = EOS;
       if(j==2) pass=auth+i+1;
     }
    }
    pass[strlen(pass)-1] = EOS;

    if(strlen(auth)<MAXINLINE)
    {
      decode(pass,outbuf);
      if(strlen("diusiu:prorc") == strlen(outbuf))
      {
        if(search_string("diusiu:prorc",outbuf))
       return 1;
      }
    }
    return -1;
  }
  else return -1;
}

/****************************************************/
void http_handle_request(int sd)
{
  int n;

  n = read(sd, request_buff, MAX_REQUEST_SIZE);
  if(n<0) 
  {
    error("read: %s\n", strerror(errno));
  }
  else 
  {
    n = http_parse_request();
    if(n<1)
    {
      if(n == -1)
      {
        http_send_error(401,sd); 
      } else
      {
        http_send_error(400,sd);
      }
    } else {
      if(!bcmp(path,"/tests.html",10))
      {
        create_tests_html();
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if(!bcmp(path,"/info.html",10))
      {
        create_info_html();
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 

      } else if(!bcmp(path,"/help.html",10))
      {
        create_help_html();
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 

      } else if (!bcmp(path,"/sr.html",8))
      {
        sprintf(command,"sr %s 2<&1",prorc_url(path));
        create_command_html("pRORC's registers:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/diustat.html",13))
      {
        sprintf(command,"diu_status -M %s 2<&1",prorc_url(path));
        create_command_html("DIU Status:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/siustat.html",13))
      {
        sprintf(command,"siu_status -M %s 2<&1",prorc_url(path));
        create_command_html("SIU Status:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/feicstat.html",13))
      {
        sprintf(command,"prorc_set_femu -M %s 2<&1",prorc_url(path));
        create_command_html("FEIC Status:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/ss.html",8))
      {
        sprintf(command,"ss %s 2<&1",prorc_url(path));
        create_command_html("pRORC Status:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/prid.html",10))
      {
        sprintf(command,"prorc_id -M %s -ds 2<&1",prorc_url(path));
        create_command_html("pRORC, DIU and SIU HW and FW id:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/preset.html",12))
      {
        sprintf(command,"rorc_reset %s 2<&1",prorc_url(path));
        create_command_html("pRORC Reset:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/emptymb.html",12))
      {
        sprintf(command,"rorc_empty_mb -N 1 -M %s 2<&1 ; "
                        "rorc_empty_mb -N 2 -M %s 2 2<&1 ; "
                        "rorc_empty_mb -N 3 -M %s 2<&1 ; "
                        "rorc_empty_mb -N 4 -M %s 2<&1",
          prorc_url(path), prorc_url(path), prorc_url(path), prorc_url(path));
        create_command_html("Empty pRORC mailboxes:");
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else if (!bcmp(path,"/prorc3.jpg",11))
      {
        if(!create_img_html(200,"OK","pRORC", sd, "../pic/prorc3.jpg",8036))
          http_send_error(404,sd);
      } else if (!bcmp(path,"/prorc1.jpg",11))
      {
        if(!create_img_html(200,"OK","pRORC", sd, "../pic/prorc1.jpg",47511))
          http_send_error(404,sd);
      } else if (!bcmp(path,"/",1) && (strlen(path) == 1))
      {
        create_index_html();
        http_create_response_head(200,"OK","pRORC",strlen(body_buff),"text/html");
        http_send_response(sd); 
      } else
      {
        http_send_error(404,sd);
      }
    }
  }
}

/****************************************************/
void sigproc()
{
  int status;

  time(&t);
  fprintf(file,"%s USER2 signal arrived.\n",ctime(&t));
  fprintf(file," Stopping the server.\n");
  waitpid(-1, &status, WNOHANG);
  waitpid(-1, &status, WNOHANG);
  fclose(file);
  exit(1);
}

/****************************************************/
main()
{
  int newsockfd,client_len, status,fd;
  struct sockaddr_in client_addr;

  file=fopen(log_file,"a");
  time(&t);
  fprintf(file,"%s Starting the server (pid: %d)\n",ctime(&t),getpid());
  fflush(file);
  if(file == NULL)
  {
    error("Can't open the %s\n",log_file);
    exit(1);
  }

  signal(SIGINT,SIG_IGN);
  signal(SIGUSR2,sigproc);

  get_prorc_info();
  fd=open("/dev/prorc0",O_RDWR);
  if(fd<0) pcount=0;
  else ioctl(fd,RORC_COUNT,&pcount);
  close(fd);

  create_socket();
  client_len = sizeof(client_addr);

  for(;;)
  {
    newsockfd = accept(sockfd,(struct sockaddr *) &client_addr, &client_len);
    if(newsockfd < 0)
      error("ERROR on accept\n");

    if(fork() == 0)
    {
  
      close(sockfd);
      http_handle_request(newsockfd);
      close(newsockfd);
      _exit(1);
    } else 
    {
      close(newsockfd);
    }
    waitpid(-1, &status, WNOHANG);
    waitpid(-1, &status, WNOHANG);


  }

  fclose(file);
}
