/***************************************************************
* rorc_arm_feic.c                                              *
* set (and read) FEE Emulator Card (FEMU) parameters           *
* Written by E. Denes                                          *
* Last modified on 28/06/2004                                  *
*                                                              *
* vers.4.3    28-Jun_2004 new event length values              *
****************************************************************/
#define SOURCE_FILE "rorc_arm_feic.c"
#define DESCRIPTION "Set FEIC registers"

#include "rorc_ddl.h"

#define FILE_NAME "feic.cfg"
  int minor = 0;
  int channel = 0;
  int TimeOut, usTimeOut = DDL_RESPONSE_TIME;

/*************************************************************/
void print_usage(char *s)
{
   printf("\n------------------------------------------------------------------------------\n");
   printf("Usage: %s [-{M|m} <minor>] [-{C|c} <channel>] [-{F|f} <file_name>]\n", s);
   printf("where\n");
   printf("minor    : RORC card number (default: %d)\n", minor);
   printf("minor    : RORC channel number (default: %d)\n", channel);
   printf("file_name: name of the FEIC configuration file. (default: %s)\n",
                           FILE_NAME);
   printf("------------------------------------------------------------------------------\n\n");

   exit (-1);
}


/****************************************************************************/
int main (int argc, char **argv)
{

  int c;
  char cfg_file_name[80];
  FILE *fp;
  int pattern, evlen, trig, flctrl, trdis, seed;
  rorc_pci_dev_t prorc;
  int retval, real_len, rounded_len;

  /* treat parameters */

  strcpy(cfg_file_name, FILE_NAME); 
  if(argc >1)
    while((c = getopt(argc, argv, "M:m:C:c:F:f:")) != EOF)
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); break;
        case 'C': channel = atoi(optarg);
          if ((channel < 0) || (channel >= MAX_CHANNEL))
          {
            printf(" Channel number should be positive and less than %d\n",
                                MAX_CHANNEL);
            print_usage (argv[0]);
          }
          break;
        case 'F': strcpy(cfg_file_name, optarg); break;
        default : print_usage(argv[0]); break;
      }

  /* open configuration file */
  fp = fopen(cfg_file_name, "r");
  if (fp == NULL)
  {
    printf(" Can not open %s file.\n", cfg_file_name);
    exit (-1);
  }

  fscanf(fp, "set patgen       = %d\n"
             "set evlen        = %x\n"
             "set trig         = %d\n"
             "set flctrl       = %d\n"
             "set trdis        = %d\n"
             "set seed         = %x\n",
              &pattern, &evlen, &trig, &flctrl, &trdis, &seed);
  //            &pattern, &evlen, &trig, &flctrl, &trdis);

  printf(" patgen = %d, evlen = 0x%x, trig = %d, flctrl = %d, trdis = %d, seed = 0x%x\n", 
           pattern, evlen, trig, flctrl, trdis, seed);
  real_len = 1 << ((evlen & 0x1f) + 3);
  printf(" length value = %d words\n", real_len);
  if ((evlen & 0x80) && seed)
    printf(" Random event length generation is set.\n");

  /* open RORC and map mailboxes */
  if (rorcOpenChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);
  TimeOut = usTimeOut * prorc.loop_per_usec;

  retval = rorcArmFeic(&prorc, pattern, real_len, trig, flctrl, trdis, seed, 
                                  TimeOut, &rounded_len);
  if (retval == RORC_LINK_NOT_ON)
    printf(" LINK IS DOWN, FEIC can not be armed\n");
  else if (retval == RORC_TIMEOUT)
    printf(" FEE command can not be sent in timeout %d usec\n", usTimeOut);
  else if (retval == RORC_STATUS_ERROR)
    printf(" rorcArmFeic returned with status error\n");
  else
    printf(" OK\n");

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);
}
