/***********************************************************
 * rorc_find.c
 *
 *  Find a RORC device by its version and/or serial numbers
 *
 * last updated: 17/11/2004
 * written by: Peter Csato and Ervin Denes
 ***********************************************************/
#define SOURCE_FILE "rorc_find.c"
#define DESCRIPTION "Find RORC devices"

#include <ctype.h>
#include "rorc_ddl.h"

/*******************************************************/
void usage(char *name)
{
  printf("Find all RORC devices not under use.\n");
  printf("usage: %s\n", name);
}

/********************************************************/
int main(int argc, char **argv)
{
  int i, version, subversion, serial;
  int max_found, found, k_found, last_minor;
  rorc_pci_dev_t prorc;
  rorcChannelId_t channel[MAX_DEVICE * MAX_CHANNEL];
  rorcHwSerial_t hw[MAX_DEVICE * MAX_CHANNEL], diu_hw[MAX_DEVICE * MAX_CHANNEL];
  int prorc_rev[MAX_DEVICE * MAX_CHANNEL], diu_vers[MAX_DEVICE * MAX_CHANNEL];

  if (system("/sbin/lsmod | grep rorc_driver > /dev/null"))
  {
    printf("RORC card not plugged or driver not loaded\n");
    exit (EXIT_FAILURE);
  }

  version = -1;
  subversion = -1;
  serial = -1;
 
  /* Check parameters */

  if (argc > 1)
  {
     usage (argv[0]); 
     exit (EXIT_FAILURE);
  }

  max_found = rorcFindAll(hw, diu_hw, channel, prorc_rev, diu_vers, MAX_DEVICE);

  found = 0;
  last_minor = -1;

  for (i = 0; i < max_found; i++)
  {
    if (!found)
    {
      printf("\n The following device(s) found:\n");
      printf(" -------------------------------------------------------------------------------\n");
      printf(" Minor Channel Device type and HW identification\n");
      printf(" -------------------------------------------------------------------------------\n");
    }
    found++;

    if (channel[i].minor != last_minor)
    {
      printf("  %2d      %2d   %s %s\n               ", 
              channel[i].minor, channel[i].channel, 
              prorc_rev[i] == -1      ? "undefined RORC " :
              prorc_rev[i] == PRORC   ? "pRORC          " :
              prorc_rev[i] == DRORC   ? "D-RORC         " :
              prorc_rev[i] == INTEG   ? "integ. D-RORC  " :
              prorc_rev[i] == DRORC2  ? "integ. DRORC2  " :
                                        "unknown RORC   " ,
              hw[i].data);
      last_minor = channel[i].minor;
    }
    else
    {
      printf("          %2d   ", channel[i].channel) ;
    }
      printf("%s %s\n", 
              diu_vers[i] == NOT_DEF  ? "undefined DIU  " :
              diu_vers[i] == NO_DIU   ? "no DIU         " :
              diu_vers[i] == OLD      ? "old DIU        " :
              diu_vers[i] == NEW      ? "new DIU        " :
              diu_vers[i] == EMBEDDED ? "embedded DIU   " :
                                        "unknown  DIU   " ,
              diu_hw[i].data);
  }
  if (found)
  {
        printf(" -------------------------------------------------------------------------------\n");
    printf("\n %d RORC channel(s) not in use was found.\n", found);
    rorcMapChannel(&prorc, channel[0].minor, channel[0].channel);
    ioctl(prorc.fd, RORC_COUNT, &k_found);
    rorcClose(&prorc);
    printf(" RORC driver reported %d RORC device(s).\n\n", k_found);
  }
  else
    printf("\n No UNUSED RORC device was found.\n\n"); 
  exit (0);

}
