/****************************************************
 * rorc_reset.c
 *
 *  General pRORC reset  
 *
 * last updated: 22/08/2003
 * written by: Peter Csato and Ervin Denes
 ****************************************************/
#define SOURCE_FILE "rorc_reset.c"
#define DESCRIPTION "General RORC reset"

#include <ctype.h>
#include "rorc_lib.h"

/******************************************************************/
void usage(char *name)
{
  printf("USAGE: %s [-{M|m} <minor>] [-{C|c} <channel>]\n", name);
  printf("                   [-D|d|B|b|S|s|F|f|O|o|E|e|N|n]\n");
  printf(" -M <minor> or -m <minor>       RORC number, default = 0\n");
  printf(" -C <channel> or -c <channel>   RORC's channel number, default = 0\n");
  printf(" The following reset commands are possible:\n");
  printf(" -D or -d: reset DIU\n");
  printf(" -B or -b: reset both RORC and DIU\n");
  printf(" -S or -s: reset SIU\n");
  printf(" -F or -f: clear Free FIFO\n");
  printf(" -O or -o: clear RORC's other FIFOs\n");
  printf(" -E or -e: clear RORC's error bits\n");
  printf(" -N or -n: clear RORC's byte counters\n");
  printf(" Missing reset code: pRORC only\n");
}

/******************************************************************/

int main(int argc, char **argv)
{
  int c, minor, channel;
  rorc_pci_dev_t prorc;
  __u32 cmd;

  minor = 0;
  channel = 0;
  cmd = 0;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:C:c:DdBbSsFfOoEeNn")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); break;
        case 'C': channel = atoi(optarg);
          if ((channel < 0) || (channel >= MAX_CHANNEL))
          {
            printf(" Channel number should be positive and less than %d\n",
                                MAX_CHANNEL);
            usage (argv[0]);
            exit (EXIT_FAILURE);
          }
          break;
        case 'D': cmd = RORC_RESET_DIU; break;
	case 'B': cmd = RORC_RESET_RORC | RORC_RESET_DIU; break;
	case 'S': cmd = RORC_RESET_SIU; break;
        case 'F': cmd = RORC_RESET_FF; break;
        case 'O': cmd = RORC_RESET_FIFOS; break;
        case 'E': cmd = RORC_RESET_ERROR; break;
        case 'N': cmd = RORC_RESET_COUNTERS; break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }

  /* open RORC and map mailboxes */
  if (rorcMapChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);

  rorcReset(&prorc, cmd);
  printf("RORC reset OK\n");

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);
}

