/* rorc_version.h */

/* software driver version number */ 
#define RORC_DRIVER_VERSION "v5.1.3"
#define RORC_DRIVER_MAJOR 5
#define RORC_DRIVER_MINOR 1
#define RORC_DRIVER_RELEASE 3

/* allowed firmware version numbers */
#define PRORC_VERSION_MIN 174
#define PRORC_VERSION_MAX 0
#define DRORC_CH1_VERSION_MIN 111
#define DRORC_CH1_VERSION_MAX 0
#define DRORC_CH2_VERSION_MIN 111
#define DRORC_CH2_VERSION_MAX 0

#ifdef SOURCE_FILE
/*****************************************************************************/
/* The following is used to identify the module (see "man what" for details) */
/* variables SOURCE_FILE and                                                 */
/*           DESCRIPTION must be defined before include this file            */
char rorcIdent[]="@(#)""" SOURCE_FILE """: """ DESCRIPTION \
     """ """ RORC_DRIVER_VERSION """ compiled """ __DATE__ """ """ __TIME__;
/*****************************************************************************/
#endif

/*
 *
 * New in version 5.1.3 (11-Aug-2006)
 *        modifications for similar SLC3/SLC4 installation procedure
 *
 * New in version 5.1.2 (11-Aug-2006)
 *        adding license information to rorc_driver
 *        creating install_rorc_driver, load_rorc_driver and 
 *                                             reload_rorc_driver scripts
 *        modifying GNUmakefile and Makefile by calling the above scripts
 *
 * New in version 5.1.1 (27-Jul-2006) 
 *        new Makefile and GNUmakefile working under both SLC3 and SLC4
 *        modified driver for SLC4
 *        fixing minor error in reset sequence of rorc_receive
 *
 * New in version 4.4.6 (26-Jun-2006)
 *        rorc_receive prints block number on output file
 *        New program: drorc_flash_programmer for DRORC remote configuration
 *        FeC2 can identify the RORC device using revision and serial numbers
 *        FeC2 accepts environment variables in the script
 *
 * New in version 4.4.5 (02-May-2006)
 *        correcting the printout of the stop time in ddg.c
 *        accept revision number 4 (DRORC2)
 *        sleep after loop-back settings in rorc_receive program
 *        check time-out option in stand-alone programs
 *        delete lines not in use from GNUmakefile and Makefile 
 *        changing diagnostic printouts in ddlSendData() ddlReadDataBlock() 
 *        additional prints of RORC Hw and Fw id in rorc_status program
 *
 * New in version 4.4.4 (05-Jan-2006)
 *        new FeC2 command: write_block_multiple
 *        clear SIU/DIU status after reset on rorc_receive program
 *        print sw version at the beginning of FeC2 and rorc_receive runs
 *        changing DTSTW register check to data transmit ending check  
 *        fixing minor code problem in rorc_aux.c 
 *        fixing minor code problem in rorc_set_femu.c 
 *        removing -static option from the compilation of ddg program
 *
 * New in version 4.4.3 (05-Sep-2005)
 *        Allow DDG to use non-integrated D-RORC in case of sw generation
 *        Add new routines checking DTSTW register emptyness
 *        Add read_ddl_status command to FeC2
 *        Add examples
 *        Checking Rx DMA count in ddlReadDataBlock()
 *        Checking Tx DMA count in ddlSendData()
 *        Infinite wait time for rorc_send
 *        New DDL commands for test mode
 *        New commands for SIU RX loopback
 *        Add text "Error" to each error messages
 *        Change variable name diu_version to siu_version in siu_status.c
 *
 * New in version 4.4.2 (16-Apr-2005) 
 *        Fix a bug in GNUmakefile
 *        Ommit "nobody" from makefiles
 *
 * New in version 4.4.1 (06-Apr-2005) 
 *        Include ddg in rorc sw package
 *        Use of physmem package version 3.1
 *        rorc_driver.o and KOPT entries removed form the OBJS list of 
 *                                                          the GNUmakefile
 *        FeC2' physmem offset depends on minor and channel number
 *
 * New in version 4.3.14 (18-Mar-2005) 
 *        fix time-out bug in ddlReadDataBlock()
 *        FeC2 stops if protocol error occurs
 *
 * New in version 4.3.13 (10-Mar-2005) 
 *        FeC2 sets failed code if too much/less data arrive
 *        for the READ_AND_CHECK_BLOCK command
 *
 * New in version 4.3.13 (03-Mar-2005) 
 *        FeC2 prints a warning if FEE command or address is too large
 *
 * New in version 4.3.13 (22-Feb-2005)
 *        introduce Rx and Tx DMA count and DMA address registers
 *        modifying rorc_reg program accordingly
 *
 * New in version 4.3.12 (08-Feb-2005)
 *        fix a bug in ddlWriteDataBlock() routine 
 *
 * New in version 4.3.11 (31-Jan-2005)
 *        no use of strtok in token browse routine 
 *        new options -w and -y for rorc_receive program
 *        change variable VERSION to RORC_DRIVER_VERSION
 *        do not reset RORC in ddlFindDiuVersion
 *                       routine in case of INTEG RORC
 *        use usec for working indicator (spinning '\') and 
 *                       print page or GB moved together with the indicator
 *
 * New in version 4.3.10: (13-Dec-2004)
 *        -I${DATE_BANKS_MANAGER_DIR}  added to COPTS in GNUmakefile
 *
 * New in version 4.3.9: (10-Dec-2004)
 *        module identification string for what command is not static any more
 *        rorc_id prints the compiled sw version
 *        rorc_find, rorcMapChannel and rorcCheckOpen returns with error
 *                          if /sbin/lsmod does not shows rorc_driver
 *
 * New in version 4.3.8: (20-Oct-2004)
 *        Fixing bug in the HW id check 
 *
 * New in version 4.3.7: (19-Oct-2004)
 *        Correcting GNUMakefile 
 *
 * New in version 4.3.6: (19-Oct-2004)
 *        Use upper case letters for HW id check
 *
 * New in version 4.3.5: (06-Aug-2004)
 *        In GNUMakefile -I/date/physmem replaced by -I${DATE_PHYSMEM_DIR}
 *        Wait 10 ms after SIU reset in ddlResetSiu() both for pRorc and D-RORC
 *        Calibrate loop time in rorcMapChannel 
 *        rorc_GUI.tcl added to the sw package
 *
 * New in version 4.3.4:
 *        Status interpreting texts to rorc_lib.h
 *
 * New in version 4.3.3:
 *        Adding HLT flow-control on/off command
 *
 * New in version 4.3.2:
 *        Changing path pRorc to rorc 
 * 
 * New in version 4.3.1:
 *        Checking RORC firmware version
 *        Changing pRorcFind
 *        Changing some routine names: instead of "pRorc",
 *                                 they should start with "rorc"
 *        Allow in-line command in FeC2 script
 *        New FeC2 command: WAIT <usecs>
 *        Fix bug in siu_reset program
 *        Changing D-RORC Hw id format
 *        Adding HLT splitter on/off command
 */
