/****************************************************
 * siu_loopback.c
 *
 *  Set / clear SIU loopback
 *
 * last updated: 14/06/2004
 * written by: Peter Csato and Ervin Denes
 ****************************************************/
#define SOURCE_FILE "siu_loopback.c"
#define DESCRIPTION "Set / clear SIU loopback"

#include <ctype.h>
#include "rorc_ddl.h"

#define SIU_LOOPBACK 1 

/******************************************************************/
void usage(char *name)
{
  printf("USAGE: %s [-{M|m} <minor>] [-{C|c} <channel>]\n",
		                      name);
  printf("                    [-{0|1}] [-{T|t} <time-out>]\n");
  printf(" -M <minor>   : RORC number, default = 0\n");
  printf(" -C <channel> : RORC channel, default = 0\n");
  printf(" -1           : set SIU loopback mode. This is the default.\n");
  printf(" -0           : clear SIU loopback mode\n");
  printf(" -T <time-out>: wait time in usec for command sending and reply, default = %d\n",
         DDL_RESPONSE_TIME);
}

/******************************************************************/

int main(int argc, char **argv)
{
  int c, minor, channel, ustime, time, siu_loopback;
  rorc_pci_dev_t prorc;
  unsigned long ret;
  stword_t stw;
  unsigned long status;

  minor = 0;
  channel = 0;
  siu_loopback = SIU_LOOPBACK;
  ustime = DDL_RESPONSE_TIME;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:C:c:10T:t:")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); break;
        case 'C': channel = atoi(optarg);
          if ((channel < 0) || (channel >= MAX_CHANNEL))
          {
            printf(" Channel number should be positive and less than %d\n",
                                                                MAX_CHANNEL);
            usage (argv[0]);
            exit (EXIT_FAILURE);
          }
          break;
        case '1': siu_loopback = 1; break;
        case '0': siu_loopback = 0; break;
        case 'T': ustime = atoi(optarg); break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }

  /* open RORC and map mailboxes */
  if (rorcOpenChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);
  time = ustime * prorc.loop_per_usec;

  /* send loopback command to SIU */
  if (siu_loopback)
    ret = ddlSetSiuLoopBack(&prorc, time, &stw);
  else
    ret = ddlClearSiuLoopBack(&prorc, time, &stw);
  if (ret != RORC_STATUS_OK)
    printf("Error: SIU loopback returned error %s\n", 
                                            rorcInterpretReturnCode(ret));
  else
  {
    if (siu_loopback)
      printf("SIU loopback mode set. SIU status:\n");
    else
      printf("SIU loopback mode cleared. SIU status:\n");
  }

  /* send DDL command for SIU status*/
  status = ddlReadSiu(&prorc, 0, time);

  /* read and interpret the answer */
  if (status != -1)
  {
    printf(" SIU status word: 0x%08lx, i.e.\n", status);
    ddlInterpretIFSTW(&prorc, status);
  }

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);
}

