/****************************************************
 * siu_reset.c
 *
 *  General SIU reset  
 *
 * last updated: 05/08/2004
 * written by: Peter Csato and Ervin Denes
 ****************************************************/
#define SOURCE_FILE "siu_reset.c"
#define DESCRIPTION "General SIU reset"

#include <ctype.h>
#include "rorc_ddl.h"

#define MAX_CYCLE 3

/******************************************************************/
void usage(char *name)
{
  printf("USAGE: %s [-{M|m} <minor>] [-{C|c} <channel>]\n",
		                      name);
  printf("                 [-{N|n} <cycle>] [-{T|t} <time-out>]\n");
  printf(" -M <minor>   : RORC number, default = 0\n");
  printf(" -C <channel> : RORC channel, default = 0\n");
  printf(" -N <cycle>   : number of DIU and SIU status check after SIU reset, default = %d\n",
          MAX_CYCLE);
  printf(" -T <time-out>: wait time in usec for command sending and reply, default = %d\n",
         DDL_RESPONSE_TIME);
}

/******************************************************************/

int main(int argc, char **argv)
{
  int c, minor, channel, ustime, time, cycle;
  rorc_pci_dev_t prorc;
  unsigned long ret;

  minor = 0;
  channel = 0;
  cycle = MAX_CYCLE;
  ustime = DDL_RESPONSE_TIME;
  if (argc > 1)
  {
    while ((c = getopt(argc, argv, "M:m:C:c:N:n:T:t:")) != EOF)
    {
      switch(toupper(c))
      {
        case 'M': minor = atoi(optarg); break;
        case 'C': channel = atoi(optarg);
          if ((channel < 0) || (channel >= MAX_CHANNEL))
          {
            printf(" Channel number should be positive and less than %d\n",
                                                                MAX_CHANNEL);
            usage (argv[0]);
            exit (EXIT_FAILURE);
          }
          break;
        case 'N': cycle = atoi(optarg); break;
        case 'T': ustime = atoi(optarg); break;
        default : usage (argv[0]); exit (EXIT_FAILURE); break;
      }
    }
  }

  /* open RORC and map mailboxes */
  if (rorcOpenChannel(&prorc, minor, channel))
    exit (EXIT_FAILURE);
  time = ustime * prorc.loop_per_usec;

  if (ddlResetSiu(&prorc, 1, cycle, time) == -1)
    printf("Unsuccessful SIU reset\n");
  else if (ret == 0)
    printf("SIU reset sent. Please check DIU and SIU status.\n");
  else
    printf("SIU reset OK\n");

  /* unmap and close RORC */
  rorcClose(&prorc);

  exit (0);
}

